@@ -1,6 +1,6 @@
 /* font-manager-menu-provider.h
  *
- * Copyright (C) 2018 - 2020 Jerry Casiano
+ * Copyright (C) 2018 - 2021 Jerry Casiano
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -24,12 +24,14 @@
 
 #include <glib.h>
 #include <glib-object.h>
+#include <glib/gi18n-lib.h>
 #include <libnemo-extension/nemo-menu-provider.h>
 
+#include "font-manager-utils.h"
+
 G_BEGIN_DECLS
 
 #define FONT_MANAGER_TYPE_MENU_PROVIDER (font_manager_menu_provider_get_type())
-
 G_DECLARE_FINAL_TYPE (FontManagerMenuProvider, font_manager_menu_provider, FONT_MANAGER, MENU_PROVIDER, GObject)
 
 void font_manager_menu_provider_load (GTypeModule *module);
