@@ -1,6 +1,6 @@
 /* nemo-font-manager-module.c
  *
- * Copyright (C) 2019 - 2020 Jerry Casiano
+ * Copyright (C) 2019 - 2021 Jerry Casiano
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -26,6 +26,8 @@
 void
 nemo_module_initialize (GTypeModule *module)
 {
+    bindtextdomain(PACKAGE_NAME, NULL);
+    bind_textdomain_codeset(PACKAGE_NAME, NULL);
     font_manager_menu_provider_load(module);
     return;
 }
