@@ -1,6 +1,6 @@
 /* font-manager-font-model.c
  *
- * Copyright (C) 2009 - 2020 Jerry Casiano
+ * Copyright (C) 2009 - 2021 Jerry Casiano
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,20 +18,25 @@
  * If not, see <http://www.gnu.org/licenses/gpl-3.0.txt>.
 */
 
-#include <gtk/gtk.h>
-#include <json-glib/json-glib.h>
-
-#include "font-manager-font.h"
-#include "font-manager-family.h"
 #include "font-manager-font-model.h"
 
+/**
+ * SECTION: font-manager-font-model
+ * @short_description: Font data model
+ * @title: Font Model
+ * @include: font-manager-font-model.h
+ * @see_also: #FontManagerFamily #FontManagerFont
+ *
+ * Minimal implementation which wraps the #JsonArray returned by #font_manager_sort_json_font_listing().
+ *
+ * This model provides read-only access to available #FontManagerFamily objects.
+ */
+
 struct _FontManagerFontModel
 {
     GObject parent_instance;
 
     gint stamp;
-    FontManagerFont *font;
-    FontManagerFamily *family;
     JsonArray *available_fonts;
 };
 
@@ -60,11 +65,43 @@ GType COLUMN_TYPES [FONT_MANAGER_FONT_MODEL_N_COLUMNS] = {
     G_TYPE_INT
 };
 
+GType
+font_manager_font_model_column_get_type (void)
+{
+  static volatile gsize g_define_type_id__volatile = 0;
+
+  if (g_once_init_enter (&g_define_type_id__volatile))
+    {
+      static const GEnumValue values[] = {
+        { FONT_MANAGER_FONT_MODEL_OBJECT, "FONT_MANAGER_FONT_MODEL_OBJECT", "object" },
+        { FONT_MANAGER_FONT_MODEL_NAME, "FONT_MANAGER_FONT_MODEL_NAME", "name" },
+        { FONT_MANAGER_FONT_MODEL_DESCRIPTION, "FONT_MANAGER_FONT_MODEL_DESCRIPTION", "description" },
+        { FONT_MANAGER_FONT_MODEL_COUNT, "FONT_MANAGER_FONT_MODEL_COUNT", "count" },
+        { FONT_MANAGER_FONT_MODEL_N_COLUMNS, "FONT_MANAGER_FONT_MODEL_N_COLUMNS", "n-columns" },
+        { 0, NULL, NULL }
+      };
+      GType g_define_type_id =
+        g_enum_register_static (g_intern_static_string ("FontManagerFontModelColumn"), values);
+      g_once_init_leave (&g_define_type_id__volatile, g_define_type_id);
+    }
+
+  return g_define_type_id__volatile;
+}
+
 #define SOURCE "source-array"
-#define JSON_OBJECT(o) (JsonObject *) o
-#define GET_INDEX(o) ((gint) json_object_get_int_member(JSON_OBJECT(o), "_index"))
-#define GET_VARIATIONS(o) json_object_get_array_member(JSON_OBJECT(o), "variations")
-#define N_VARIATIONS(o) ((gint) json_object_get_int_member(JSON_OBJECT(o), "n_variations"))
+
+gint
+get_n_variations (FontManagerFontModel *self, gint index)
+{
+    JsonObject *family = json_array_get_object_element(self->available_fonts, index);
+    return (gint) json_object_get_int_member(family, "n_variations");
+}
+
+gboolean
+invalid_iter (GtkTreeIter *iter) {
+    iter->stamp = 0;
+    return FALSE;
+}
 
 /* GtkTreeModelIface */
 
@@ -89,22 +126,25 @@ font_manager_font_model_get_column_type (G_GNUC_UNUSED GtkTreeModel *tree_model,
 
 static gboolean
 font_manager_font_model_get_iter (GtkTreeModel *tree_model,
-                                 GtkTreeIter *iter,
-                                 GtkTreePath *path)
+                                  GtkTreeIter *iter,
+                                  GtkTreePath *path)
 {
     FontManagerFontModel *self = FONT_MANAGER_FONT_MODEL(tree_model);
     g_return_val_if_fail(self != NULL, FALSE);
     g_return_val_if_fail(path != NULL, FALSE);
     gint depth = gtk_tree_path_get_depth(path);
     gint *indices = gtk_tree_path_get_indices(path);
+    if (!self->available_fonts || indices[0] >= ((int) json_array_get_length(self->available_fonts)))
+        return invalid_iter(iter);
     iter->stamp = self->stamp;
-    iter->user_data = json_array_get_object_element(self->available_fonts, indices[0]);
-    g_return_val_if_fail(iter->user_data != NULL, FALSE);
+    iter->user_data = GINT_TO_POINTER(indices[0]);
+    iter->user_data2 = GINT_TO_POINTER(-1);
     if (depth > 1) {
-        JsonArray *variations = GET_VARIATIONS(iter->user_data);
-        iter->user_data2 = json_array_get_object_element(variations, indices[1]);
-    } else
-        iter->user_data2 = NULL;
+        gint n_variations = get_n_variations(self, indices[0]);
+        if (indices[1] >= n_variations)
+            return invalid_iter(iter);
+        iter->user_data2 = GINT_TO_POINTER(indices[1]);
+    }
     return TRUE;
 }
 
@@ -114,29 +154,35 @@ font_manager_font_model_get_path (GtkTreeModel *tree_model, GtkTreeIter *iter)
     FontManagerFontModel *self = FONT_MANAGER_FONT_MODEL(tree_model);
     g_return_val_if_fail(self != NULL, NULL);
     g_return_val_if_fail(iter->stamp == self->stamp, NULL);
-    g_return_val_if_fail(iter->user_data != NULL, NULL);
-    gint index = GET_INDEX(iter->user_data);
-    if (iter->user_data2 == NULL)
-        return gtk_tree_path_new_from_indices(index, -1);
-    else
-        return gtk_tree_path_new_from_indices(index, GET_INDEX(iter->user_data2), -1);
+    GtkTreePath *path = gtk_tree_path_new();
+    gtk_tree_path_append_index(path, GPOINTER_TO_INT(iter->user_data));
+    if (GPOINTER_TO_INT(iter->user_data2) != -1)
+        gtk_tree_path_append_index(path, GPOINTER_TO_INT(iter->user_data2));
+    return path;
 }
 
 static void
 font_manager_font_model_get_value (GtkTreeModel *tree_model,
-                                  GtkTreeIter *iter,
-                                  gint column,
-                                  GValue *value)
+                                   GtkTreeIter *iter,
+                                   gint column,
+                                   GValue *value)
 {
     FontManagerFontModel *self = FONT_MANAGER_FONT_MODEL(tree_model);
     g_return_if_fail(self != NULL);
+    g_return_if_fail(self->available_fonts != NULL);
+    g_return_if_fail(json_array_get_length(self->available_fonts) > 0);
     g_return_if_fail(iter != NULL);
     g_return_if_fail(iter->stamp == self->stamp);
-    g_return_if_fail(iter->user_data != NULL);
     g_value_init(value, COLUMN_TYPES[column]);
-    gboolean root_node = (iter->user_data2 == NULL);
+    JsonObject *root = NULL, *child = NULL;
+    root = json_array_get_object_element(self->available_fonts, GPOINTER_TO_INT(iter->user_data));
+    gboolean root_node = (GPOINTER_TO_INT(iter->user_data2) == -1);
+    if (!root_node) {
+        JsonArray *children = json_object_get_array_member(root, "variations");
+        child = json_array_get_object_element(children, GPOINTER_TO_INT(iter->user_data2));
+    }
+    JsonObject *obj = root_node ? root : child;
     const gchar *member = root_node ? "family" : "style";
-    JsonObject *obj = root_node ? JSON_OBJECT(iter->user_data) : JSON_OBJECT(iter->user_data2);
     switch (column) {
         case FONT_MANAGER_FONT_MODEL_NAME:
             g_value_set_string(value, json_object_get_string_member(obj, member));
@@ -146,17 +192,19 @@ font_manager_font_model_get_value (GtkTreeModel *tree_model,
             break;
         case FONT_MANAGER_FONT_MODEL_COUNT:
             if (root_node)
-                g_value_set_int(value, N_VARIATIONS(obj));
+                g_value_set_int(value, get_n_variations(self, GPOINTER_TO_INT(iter->user_data)));
             else
                 g_value_set_int(value, -1);
             break;
         case FONT_MANAGER_FONT_MODEL_OBJECT:
             if (root_node) {
-                g_object_set(self->family, "source-object", obj, NULL);
-                g_value_set_object(value, self->family);
+                g_autoptr(FontManagerFamily) family = font_manager_family_new();
+                g_object_set(family, "source-object", obj, NULL);
+                g_value_set_object(value, family);
             } else {
-                g_object_set(self->font, "source-object", obj, NULL);
-                g_value_set_object(value, self->font);
+                g_autoptr(FontManagerFont) font = font_manager_font_new();
+                g_object_set(font, "source-object", obj, NULL);
+                g_value_set_object(value, font);
             }
             break;
         default:
@@ -172,21 +220,20 @@ font_manager_font_model_iter_next (GtkTreeModel *tree_model, GtkTreeIter *iter)
     g_return_val_if_fail(self != NULL, FALSE);
     g_return_val_if_fail(iter != NULL, FALSE);
     g_return_val_if_fail(iter->stamp == self->stamp, FALSE);
-    g_return_val_if_fail(iter->user_data != NULL, FALSE);
-    gint index;
-    if (iter->user_data2 == NULL) {
+    if (!self->available_fonts || json_array_get_length(self->available_fonts) < 1)
+        return invalid_iter(iter);
+    gint index = GPOINTER_TO_INT(iter->user_data);
+    if (GPOINTER_TO_INT(iter->user_data2) == -1) {
         gint n_root_nodes = (gint) json_array_get_length(self->available_fonts);
-        index = GET_INDEX(iter->user_data);
         if (!(index < n_root_nodes - 1))
-            return FALSE;
-        iter->user_data = json_array_get_object_element(self->available_fonts, index + 1);
+            return invalid_iter(iter);
+        iter->user_data = GINT_TO_POINTER(index + 1);
     } else {
-        gint n_children = N_VARIATIONS(iter->user_data);
-        index = GET_INDEX(iter->user_data2);
+        gint n_children = get_n_variations(self, index);
+        index = GPOINTER_TO_INT(iter->user_data2);
         if (!(index < n_children - 1))
-            return FALSE;
-        JsonArray *variations = GET_VARIATIONS(iter->user_data);
-        iter->user_data2 = json_array_get_object_element(variations, index + 1);
+            return invalid_iter(iter);
+        iter->user_data2 = GINT_TO_POINTER(index + 1);
     }
     return TRUE;
 }
@@ -198,43 +245,42 @@ font_manager_font_model_iter_previous (GtkTreeModel *tree_model, GtkTreeIter *it
     g_return_val_if_fail(self != NULL, FALSE);
     g_return_val_if_fail(iter != NULL, FALSE);
     g_return_val_if_fail(iter->stamp == self->stamp, FALSE);
-    g_return_val_if_fail(iter->user_data != NULL, FALSE);
-    gint index;
-    if (iter->user_data2 == NULL) {
-        index = GET_INDEX(iter->user_data);
+    if (!self->available_fonts || json_array_get_length(self->available_fonts) < 1)
+        return invalid_iter(iter);
+    gint index = GPOINTER_TO_INT(iter->user_data);
+    if (GPOINTER_TO_INT(iter->user_data2) == -1) {
         if (index < 1)
-            return FALSE;
-        iter->user_data = json_array_get_object_element(self->available_fonts, index - 1);
+            return invalid_iter(iter);
+        iter->user_data = GINT_TO_POINTER(index - 1);
     } else {
-        index = GET_INDEX(iter->user_data2);
+        index = GPOINTER_TO_INT(iter->user_data2);
         if (index < 1)
-            return FALSE;
-        JsonArray *variations = GET_VARIATIONS(iter->user_data);
-        iter->user_data2 = json_array_get_object_element(variations, index - 1);
+            return invalid_iter(iter);
+        iter->user_data2 = GINT_TO_POINTER(index - 1);
     }
     return TRUE;
 }
 
 static gboolean
 font_manager_font_model_iter_children (GtkTreeModel *tree_model,
-                                      GtkTreeIter *iter,
-                                      GtkTreeIter *parent)
+                                       GtkTreeIter *iter,
+                                       GtkTreeIter *parent)
 {
     FontManagerFontModel *self = FONT_MANAGER_FONT_MODEL(tree_model);
     g_return_val_if_fail(self != NULL, FALSE);
     iter->stamp = self->stamp;
+    if (!self->available_fonts || json_array_get_length(self->available_fonts) < 1)
+        return invalid_iter(iter);
     /* Special case - if parent equals %NULL this function should return the first node */
     if (parent == NULL) {
-        iter->user_data = json_array_get_object_element(self->available_fonts, 0);
-        iter->user_data2 = NULL;
-    } else if (parent->user_data2 != NULL) {
+        iter->user_data = GINT_TO_POINTER(0);
+        iter->user_data2 = GINT_TO_POINTER(-1);
+    } else if (GPOINTER_TO_INT(parent->user_data2) != -1) {
         /* Maximum depth of this model is 2 */
-        iter->stamp = 0;
-        return FALSE;
+        return invalid_iter(iter);
     } else {
         iter->user_data = parent->user_data;
-        JsonArray *variations = GET_VARIATIONS(iter->user_data);
-        iter->user_data2 = json_array_get_object_element(variations, 0);
+        iter->user_data2 = GINT_TO_POINTER(0);
     }
     return TRUE;
 }
@@ -246,7 +292,9 @@ font_manager_font_model_iter_has_child (GtkTreeModel *tree_model, GtkTreeIter *i
     g_return_val_if_fail(self != NULL, FALSE);
     g_return_val_if_fail(iter != NULL, FALSE);
     g_return_val_if_fail(iter->stamp == self->stamp, FALSE);
-    return (iter->user_data != NULL && iter->user_data2 == NULL);
+    if (!self->available_fonts || json_array_get_length(self->available_fonts) < 1)
+        return FALSE;
+    return (GPOINTER_TO_INT(iter->user_data2) == -1);
 }
 
 static gint
@@ -254,46 +302,45 @@ font_manager_font_model_iter_n_children (GtkTreeModel *tree_model, GtkTreeIter *
 {
     FontManagerFontModel *self = FONT_MANAGER_FONT_MODEL(tree_model);
     g_return_val_if_fail(self != NULL, 0);
+    g_return_val_if_fail(self->available_fonts != NULL, 0);
     /* Special case - if iter is %NULL this function should return the number of toplevel nodes */
     if (iter == NULL)
         return ((gint) json_array_get_length(self->available_fonts));
-    g_return_val_if_fail(iter->user_data != NULL, 0);
-    return N_VARIATIONS(iter->user_data);
+    return get_n_variations(self, GPOINTER_TO_INT(iter->user_data));
 }
 
 static gboolean
 font_manager_font_model_iter_nth_child (GtkTreeModel *tree_model,
-                                       GtkTreeIter *iter,
-                                       GtkTreeIter *parent,
-                                       gint n)
+                                        GtkTreeIter *iter,
+                                        GtkTreeIter *parent,
+                                        gint n)
 {
     FontManagerFontModel *self = FONT_MANAGER_FONT_MODEL(tree_model);
     g_return_val_if_fail(self != NULL, FALSE);
     g_return_val_if_fail(n >= 0, FALSE);
-
+    if (!self->available_fonts || json_array_get_length(self->available_fonts) < 1)
+        return FALSE;
     iter->stamp = self->stamp;
     /* Special case - if parent is %NULL this function should set iter to toplevel node n */
     if (parent == NULL) {
         gint n_root_nodes = (gint) json_array_get_length(self->available_fonts);
         if (n < n_root_nodes) {
-            iter->user_data = json_array_get_object_element(self->available_fonts, n);
-            iter->user_data2 = NULL;
+            iter->user_data = GINT_TO_POINTER(n);
+            iter->user_data2 = GINT_TO_POINTER(-1);
             return TRUE;
         } else {
-            return FALSE;
+            return invalid_iter(iter);
         }
     }
 
-    g_return_val_if_fail(parent->user_data != NULL, FALSE);
-    gint n_children = N_VARIATIONS(parent->user_data);
+    g_return_val_if_fail(parent->stamp == self->stamp, FALSE);
+    gint n_children = get_n_variations(self, GPOINTER_TO_INT(parent->user_data));
 
     if (n > n_children -1) {
-        return FALSE;
+        return invalid_iter(iter);
     } else {
         iter->user_data = parent->user_data;
-        JsonArray *variations = GET_VARIATIONS(iter->user_data);
-        iter->user_data2 = json_array_get_object_element(variations, n);
-        g_return_val_if_fail(iter->user_data2 != NULL, FALSE);
+        iter->user_data2 = GINT_TO_POINTER(n);
         return TRUE;
     }
 
@@ -301,8 +348,8 @@ font_manager_font_model_iter_nth_child (GtkTreeModel *tree_model,
 
 static gboolean
 font_manager_font_model_iter_parent (GtkTreeModel *tree_model,
-                                    GtkTreeIter *iter,
-                                    GtkTreeIter *child)
+                                     GtkTreeIter *iter,
+                                     GtkTreeIter *child)
 {
     FontManagerFontModel *self = FONT_MANAGER_FONT_MODEL(tree_model);
     g_return_val_if_fail(self != NULL, FALSE);
@@ -311,7 +358,7 @@ font_manager_font_model_iter_parent (GtkTreeModel *tree_model,
     g_return_val_if_fail(child->user_data2 != NULL, FALSE);
     iter->stamp = self->stamp;
     iter->user_data = child->user_data;
-    iter->user_data2 = NULL;
+    iter->user_data2 = GINT_TO_POINTER(-1);
     return TRUE;
 }
 
@@ -338,7 +385,10 @@ font_manager_font_model_get_item (GListModel *self, guint position)
     if (position >= font_manager_font_model_get_n_items(self))
         return NULL;
     FontManagerFontModel *model = FONT_MANAGER_FONT_MODEL(self);
-    return json_array_get_object_element(model->available_fonts, position);
+    JsonObject *obj = json_array_get_object_element(model->available_fonts, position);
+    FontManagerFamily *family = font_manager_family_new();
+    g_object_set(G_OBJECT(family), "source-object", obj, NULL);
+    return family;
 }
 
 static void
@@ -382,8 +432,8 @@ font_manager_font_model_row_draggable (G_GNUC_UNUSED GtkTreeDragSource *source,
 
 static gboolean
 font_manager_font_model_drag_data_get (GtkTreeDragSource *source,
-                                      GtkTreePath *path,
-                                      GtkSelectionData *selection_data)
+                                       GtkTreePath *path,
+                                       GtkSelectionData *selection_data)
 {
     if (gtk_tree_set_row_drag_data(selection_data, GTK_TREE_MODEL(source), path))
         return TRUE;
@@ -440,39 +490,36 @@ static void
 font_manager_font_model_init (FontManagerFontModel *self)
 {
     do { self->stamp = g_random_int(); } while (self->stamp == 0);
-    self->family = font_manager_family_new();
-    self->font = font_manager_font_new();
+    self->available_fonts = json_array_new();
     return;
 }
 
 static void
-font_manager_font_model_finalize (GObject *object)
+font_manager_font_model_dispose (GObject *gobject)
 {
-    FontManagerFontModel *self = FONT_MANAGER_FONT_MODEL(object);
-    g_return_if_fail(self != NULL);
-    if (self->available_fonts)
-        json_array_unref(self->available_fonts);
-    g_clear_object(&self->family);
-    g_clear_object(&self->font);
-    G_OBJECT_CLASS(font_manager_font_model_parent_class)->finalize(object);
+    g_return_if_fail(gobject != NULL);
+    FontManagerFontModel *self = FONT_MANAGER_FONT_MODEL(gobject);
+    g_clear_pointer(&self->available_fonts, json_array_unref);
+    G_OBJECT_CLASS(font_manager_font_model_parent_class)->dispose(gobject);
     return;
 }
 
 static void
-font_manager_font_model_get_property (GObject *object,
-                                     guint property_id,
-                                     GValue *value,
-                                     GParamSpec *pspec)
+font_manager_font_model_get_property (GObject *gobject,
+                                      guint property_id,
+                                      GValue *value,
+                                      GParamSpec *pspec)
 {
-    FontManagerFontModel *self = FONT_MANAGER_FONT_MODEL(object);
-    g_return_if_fail(self != NULL && self->available_fonts != NULL);
+    g_return_if_fail(gobject != NULL);
+    FontManagerFontModel *self = FONT_MANAGER_FONT_MODEL(gobject);
+    g_return_if_fail(self->available_fonts != NULL);
 
     switch (property_id) {
         case PROP_SOURCE:
             g_value_set_boxed(value, self->available_fonts);
             break;
         default:
-            G_OBJECT_WARN_INVALID_PROPERTY_ID(object, property_id, pspec);
+            G_OBJECT_WARN_INVALID_PROPERTY_ID(gobject, property_id, pspec);
             break;
     }
 
@@ -493,20 +540,20 @@ set_source (FontManagerFontModel *self, JsonArray *value)
 }
 
 static void
-font_manager_font_model_set_property (GObject *object,
-                                     guint property_id,
-                                     const GValue *value,
-                                     GParamSpec *pspec)
+font_manager_font_model_set_property (GObject *gobject,
+                                      guint property_id,
+                                      const GValue *value,
+                                      GParamSpec *pspec)
 {
-    FontManagerFontModel *self = FONT_MANAGER_FONT_MODEL(object);
-    g_return_if_fail(self != NULL);
+    g_return_if_fail(gobject != NULL);
+    FontManagerFontModel *self = FONT_MANAGER_FONT_MODEL(gobject);
 
     switch (property_id) {
         case PROP_SOURCE:
             set_source(self, g_value_get_boxed(value));
             break;
         default:
-            G_OBJECT_WARN_INVALID_PROPERTY_ID(object, property_id, pspec);
+            G_OBJECT_WARN_INVALID_PROPERTY_ID(gobject, property_id, pspec);
             break;
     }
 
@@ -519,10 +566,18 @@ font_manager_font_model_class_init (FontManagerFontModelClass *klass)
     GObjectClass *object_class = G_OBJECT_CLASS(klass);
     object_class->get_property = font_manager_font_model_get_property;
     object_class->set_property = font_manager_font_model_set_property;
-    object_class->finalize = font_manager_font_model_finalize;
+    object_class->dispose = font_manager_font_model_dispose;
+
+    /**
+     * FontManagerFontModel:source-array:
+     *
+     * #JsonArray source.
+     */
     g_object_class_install_property(object_class,
                                     PROP_SOURCE,
-                                    g_param_spec_boxed(SOURCE, NULL, NULL,
+                                    g_param_spec_boxed(SOURCE,
+                                                       NULL,
+                                                       "#JsonArray backing this model",
                                                        JSON_TYPE_ARRAY,
                                                        G_PARAM_STATIC_STRINGS |
                                                        G_PARAM_READWRITE));
@@ -532,10 +587,8 @@ font_manager_font_model_class_init (FontManagerFontModelClass *klass)
 /**
  * font_manager_font_model_new:
  *
- * Minimal #GtkTreeModel implementation which wraps the #JsonArray
- * returned by #font_manager_sort_json_font_listing
- *
- * Returns : (transfer full) : a new #FontManagerFontModel
+ * Returns : (transfer full) : A newly created #FontManagerFontModel.
+ * Free the returned object using #g_object_unref().
  */
 FontManagerFontModel *
 font_manager_font_model_new (void)
