@@ -1,6 +1,6 @@
 /* font-manager-font-model.h
  *
- * Copyright (C) 2009 - 2020 Jerry Casiano
+ * Copyright (C) 2009 - 2021 Jerry Casiano
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -23,12 +23,27 @@
 
 #include <glib.h>
 #include <glib-object.h>
+#include <gtk/gtk.h>
+#include <json-glib/json-glib.h>
+
+#include "font-manager-font.h"
+#include "font-manager-family.h"
 
 G_BEGIN_DECLS
 
 #define FONT_MANAGER_TYPE_FONT_MODEL (font_manager_font_model_get_type())
 G_DECLARE_FINAL_TYPE(FontManagerFontModel, font_manager_font_model, FONT_MANAGER, FONT_MODEL, GObject)
 
+/**
+ * FontManagerFontModelColumn:
+ * @FONT_MANAGER_FONT_MODEL_OBJECT:         #FontManagerFont
+ * @FONT_MANAGER_FONT_MODEL_NAME:           Object name
+ * @FONT_MANAGER_FONT_MODEL_DESCRIPTION:    Font description
+ * @FONT_MANAGER_FONT_MODEL_COUNT:          Number of variations for family objects
+ * @FONT_MANAGER_FONT_MODEL_N_COLUMNS:      Number of columns in this model
+ *
+ * Index of available columns in #FontManagerFontModel.
+ */
 typedef enum
 {
     FONT_MANAGER_FONT_MODEL_OBJECT,
@@ -39,6 +54,9 @@ typedef enum
 }
 FontManagerFontModelColumn;
 
+GType font_manager_font_model_column_get_type (void) G_GNUC_CONST;
+#define FONT_MANAGER_TYPE_FONT_MODEL_COLUMN (font_manager_font_model_column_get_type ())
+
 FontManagerFontModel * font_manager_font_model_new (void);
 
 G_END_DECLS
