@@ -1,6 +1,6 @@
 /* font-manager-fontconfig.c
  *
- * Copyright (C) 2009 - 2020 Jerry Casiano
+ * Copyright (C) 2009 - 2021 Jerry Casiano
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,37 +18,20 @@
  * If not, see <http://www.gnu.org/licenses/gpl-3.0.txt>.
 */
 
-#include <stdio.h>
-#include <glib.h>
-#include <glib/gprintf.h>
-#include <glib/gstdio.h>
-#include <fontconfig/fontconfig.h>
-#include <fontconfig/fcfreetype.h>
-#include <json-glib/json-glib.h>
-#include <pango/pango-font.h>
-#include <pango/pangofc-font.h>
-#include <pango/pangofc-fontmap.h>
-#include <pango/pango-utils.h>
-#include <pango/pango-version-macros.h>
-
 #include "font-manager-fontconfig.h"
-#include "font-manager-json.h"
-#include "font-manager-utils.h"
-
-#include "unicode-info.h"
-
-#define PANGO_1_44 PANGO_VERSION_ENCODE(1, 44, 0)
 
 /**
  * SECTION: font-manager-fontconfig
  * @short_description: Fontconfig related utility functions
- * @title: FontManagerFontconfig
+ * @title: Fontconfig Utility Functions
  * @include: font-manager-fontconfig.h
- * @see_also: https://www.freedesktop.org/software/fontconfig/fontconfig-devel/
+ * @see_also:
+ * #FontManagerFont
+ * #FontManagerFamily
+ * https://www.freedesktop.org/software/fontconfig/fontconfig-devel/
  */
 
-static GList * list_charset (const FcCharSet *charset);
-static void process_fontset (const FcFontSet *fontset, JsonObject *json_obj);
+G_DEFINE_QUARK(font-manager-fontconfig-error-quark, font_manager_fontconfig_error)
 
 static const gchar *DEFAULT_VARIANTS[5] = {
     "Regular",
@@ -58,6 +41,79 @@ static const gchar *DEFAULT_VARIANTS[5] = {
     "Book"
 };
 
+static void
+set_error (const gchar *message, GError **error)
+{
+    g_return_if_fail(error == NULL || *error == NULL);
+    const gchar *msg_format = "Fontconfig Error : (%s)";
+    g_debug(msg_format, message);
+    g_set_error(error,
+                FONT_MANAGER_FONTCONFIG_ERROR,
+                FONT_MANAGER_FONTCONFIG_ERROR_FAILED,
+                msg_format, message);
+    return;
+}
+
+#define PANGO_1_44 PANGO_VERSION_ENCODE(1, 44, 0)
+/* Note : Bitmap font support was dropped in Pango 1.44 */
+gboolean
+is_legacy_format (FcPattern *pat)
+{
+    FcChar8 *format;
+    g_assert(FcPatternGetString(pat, FC_FONTFORMAT, 0, &format) == FcResultMatch);
+    return (g_strcmp0((const gchar *) format, "CFF") != 0 &&
+            g_strcmp0((const gchar *) format, "TrueType") != 0);
+}
+
+/* From pangofc-fontmap.c */
+static GList *
+list_charset (const FcCharSet *charset)
+{
+    GList *result = NULL;
+    FcChar32  ucs4, pos;
+    FcChar32  map[FC_CHARSET_MAP_SIZE];
+
+    for (ucs4 = FcCharSetFirstPage (charset, map, &pos);
+         ucs4 != FC_CHARSET_DONE;
+         ucs4 = FcCharSetNextPage (charset, map, &pos)) {
+
+        for (int i = 0; i < FC_CHARSET_MAP_SIZE; i++) {
+            int b = 0;
+            FcChar32 bits = map[i];
+            FcChar32 base = ucs4 + i * 32;
+            while (bits) {
+                if (bits & 1) {
+                    gunichar ch = (base + b);
+                    if (unicode_unichar_isgraph(ch))
+                        result = g_list_prepend(result, GINT_TO_POINTER(ch));
+                }
+                bits >>= 1;
+                b++;
+            }
+        }
+
+    }
+
+    return g_list_reverse(result);
+}
+
+static void
+process_fontset (const FcFontSet *fontset, JsonObject *json_obj)
+{
+    for (int i = 0; i < fontset->nfont; i++) {
+        if (pango_version() >= PANGO_1_44 && is_legacy_format(fontset->fonts[i]))
+            continue;
+        JsonObject *font_obj = font_manager_get_attributes_from_fontconfig_pattern(fontset->fonts[i]);
+        const gchar *family = json_object_get_string_member(font_obj, "family");
+        const gchar *style = json_object_get_string_member(font_obj, "style");
+        if (!json_object_get_member(json_obj, family))
+            json_object_set_object_member(json_obj, family, json_object_new());
+        JsonObject *family_obj = json_object_get_object_member(json_obj, family);
+        json_object_set_object_member(family_obj, style, font_obj);
+    }
+    return;
+}
+
 /**
  * font_manager_update_font_configuration:
  *
@@ -85,7 +141,7 @@ font_manager_enable_user_font_configuration (gboolean enable)
  * font_manager_add_application_font:
  * @filepath: full path to font file to add to configuration
  *
- * Equivalent to #FcConfigAppFontAddFile
+ * Equivalent to FcConfigAppFontAddFile
  *
  * Returns: %TRUE on success
  */
@@ -99,7 +155,7 @@ font_manager_add_application_font (const gchar *filepath)
  * font_manager_add_application_font_directory:
  * @dir: full path to directory to add to application specific configuration
  *
- * Equivalent to #FcConfigAppFontAddDir
+ * Equivalent to FcConfigAppFontAddDir
  *
  * Returns: %TRUE on success
  */
@@ -112,7 +168,7 @@ font_manager_add_application_font_directory (const gchar *dir)
 /**
  * font_manager_clear_application_fonts:
  *
- * Equivalent to #FcConfigAppFontClear
+ * Equivalent to FcConfigAppFontClear
  */
 void
 font_manager_clear_application_fonts (void)
@@ -125,7 +181,7 @@ font_manager_clear_application_fonts (void)
  * font_manager_load_font_configuration_file:
  * @filepath: full path to a valid fontconfig configuration file
  *
- * Equivalent to #FcConfigParseAndLoad
+ * Equivalent to FcConfigParseAndLoad
  *
  * Returns: %TRUE on success
  */
@@ -135,22 +191,12 @@ font_manager_load_font_configuration_file (const gchar *filepath)
     return FcConfigParseAndLoad(FcConfigGetCurrent(), (FcChar8 *) filepath, FALSE);
 }
 
-/* Note : Bitmap font support was dropped in Pango 1.44 */
-gboolean
-is_legacy_format (FcPattern *pat)
-{
-    FcChar8 *format;
-    g_assert(FcPatternGetString(pat, FC_FONTFORMAT, 0, &format) == FcResultMatch);
-    return (g_strcmp0((const gchar *) format, "CFF") != 0 &&
-            g_strcmp0((const gchar *) format, "TrueType") != 0);
-}
-
 /**
  * font_manager_list_available_font_files:
  *
  * Returns: (element-type utf8) (transfer full) (nullable):
  * a newly created #GSList containing filepaths or %NULL.
- * Free the returned list using #g_slist_free_full(list, g_free)
+ * Free the returned list using #g_slist_free_full(list, #g_free)
  */
 GList *
 font_manager_list_available_font_files (void)
@@ -172,7 +218,6 @@ font_manager_list_available_font_files (void)
     FcObjectSetDestroy(objectset);
     FcPatternDestroy(pattern);
     FcFontSetDestroy(fontset);
-    result = g_list_sort(result, (GCompareFunc) font_manager_natural_sort);
     return result;
 }
 
@@ -182,7 +227,7 @@ font_manager_list_available_font_files (void)
  *
  * Returns: (element-type utf8) (transfer full) (nullable):
  * a newly created #GSList containing filepaths or %NULL
- * Free the returned list using #g_slist_free_full(list, g_free);
+ * Free the returned list using #g_slist_free_full(list, #g_free);
  */
 GList *
 font_manager_list_font_directories (gboolean recursive)
@@ -208,7 +253,6 @@ font_manager_list_font_directories (gboolean recursive)
     }
 
     FcStrListDone(fdlist);
-    result = g_list_sort(result, (GCompareFunc) font_manager_natural_sort);
     return result;
 }
 
@@ -220,7 +264,7 @@ font_manager_list_font_directories (gboolean recursive)
  *
  * Returns: (element-type utf8) (transfer full) (nullable):
  * a newly created #GSList containing filepaths or %NULL
- * Free the returned list using #g_slist_free_full(list, g_free);
+ * Free the returned list using #g_slist_free_full(list, #g_free);
  */
 GList *
 font_manager_list_user_font_directories (gboolean recursive)
@@ -248,7 +292,6 @@ font_manager_list_user_font_directories (gboolean recursive)
     }
 
     FcStrListDone(fdlist);
-    result = g_list_sort(result, (GCompareFunc) font_manager_natural_sort);
     return result;
 }
 
@@ -257,7 +300,7 @@ font_manager_list_user_font_directories (gboolean recursive)
  *
  * Returns: (element-type utf8) (transfer full) (nullable):
  * a newly created #GSList containing family names or %NULL
- * Free the returned list using #g_slist_free_full(list, g_free);
+ * Free the returned list using #g_slist_free_full(list, #g_free);
  */
 GList *
 font_manager_list_available_font_families (void)
@@ -280,7 +323,6 @@ font_manager_list_available_font_families (void)
     FcObjectSetDestroy(objectset);
     FcPatternDestroy(pattern);
     FcFontSetDestroy(fontset);
-    result = g_list_sort(result, (GCompareFunc) font_manager_natural_sort);
     return result;
 }
 
@@ -291,7 +333,30 @@ font_manager_list_available_font_families (void)
  * If @family_name is not %NULL, only information for fonts belonging to
  * specified family will be returned.
  *
- * Returns: (transfer full): a newly created #JsonObject
+ * The returned #JsonObject will have the following structure:
+ *
+ * |[
+ * {
+ *   "family" : {
+ *     "variation" : {
+ *     "filepath" : string,
+ *     "findex" : int,
+ *     "family" : string,
+ *     "style" : string,
+ *     "spacing" : int,
+ *     "slant" : int,
+ *     "weight" : int,
+ *     "width" : int,
+ *     "description" : string,
+ *     },
+ *     ...
+ *   },
+ *   ...
+ * }
+ *]|
+ *
+ * Returns: (transfer full): A newly created #JsonObject which should be
+ * freed using #json_object_unref() when no longer needed.
  */
 JsonObject *
 font_manager_get_available_fonts (const gchar *family_name)
@@ -329,7 +394,14 @@ font_manager_get_available_fonts (const gchar *family_name)
  * @lang_id: should be of the form Ll-Tt where Ll is a two or three letter
  * language from ISO 639 and Tt is a territory from ISO 3166.
  *
- * Returns: (transfer full): a newly created #JsonObject
+ * See #font_manager_get_available_fonts for a description of the
+ * #JsonObject returned by this function.
+ *
+ * The returned object will contain only those fonts which claim
+ * to support @lang_id.
+ *
+ * Returns: (transfer full): A newly created #JsonObject which should be
+ * freed using #json_object_unref() when no longer needed.
  */
 JsonObject *
 font_manager_get_available_fonts_for_lang (const gchar *lang_id)
@@ -350,6 +422,7 @@ font_manager_get_available_fonts_for_lang (const gchar *lang_id)
                                               FC_WIDTH,
                                               FC_SPACING,
                                               FC_LANG,
+                                              FC_FONTFORMAT,
                                               NULL);
 
     FcFontSet *fontset = FcFontList(FcConfigGetCurrent(), pattern, objectset);
@@ -367,7 +440,14 @@ font_manager_get_available_fonts_for_lang (const gchar *lang_id)
  * font_manager_get_available_fonts_for_chars:
  * @chars: string of characters to search for
  *
- * Returns: (transfer full): a newly created #JsonObject
+ * See #font_manager_get_available_fonts for a description of the
+ * #JsonObject returned by this function.
+ *
+ * The returned object will only contain those fonts which contain all
+ * the characters in @chars.
+ *
+ * Returns: (transfer full): A newly created #JsonObject which should be
+ * freed using #json_object_unref() when no longer needed.
  */
 JsonObject *
 font_manager_get_available_fonts_for_chars (const gchar *chars)
@@ -381,6 +461,7 @@ font_manager_get_available_fonts_for_chars (const gchar *chars)
                                                FC_WIDTH,
                                                FC_SPACING,
                                                FC_CHARSET,
+                                               FC_FONTFORMAT,
                                                NULL);
 
     gunichar wc;
@@ -409,14 +490,14 @@ font_manager_get_available_fonts_for_chars (const gchar *chars)
 
 /**
  * font_manager_get_langs_from_fontconfig_pattern: (skip)
- * @pattern: #FcPattern to examine
+ * @pattern: FcPattern to examine
  *
  * Supplied FcPattern must contain an FcLangSet.
  *
  * Returns: (element-type utf8) (transfer full) (nullable):
  * a newly created #GSList or %NULL
  * The returned list contains dynamically allocated strings and should be
- * freed using #g_slist_free_full(slist, g_free) when no longer needed.
+ * freed using #g_slist_free_full(slist, #g_free) when no longer needed.
  */
 GList *
 font_manager_get_langs_from_fontconfig_pattern (FcPattern *pattern)
@@ -439,7 +520,7 @@ font_manager_get_langs_from_fontconfig_pattern (FcPattern *pattern)
 
 /**
  * font_manager_get_attributes_from_fontconfig_pattern: (skip)
- * @pattern: #FcPattern to examine
+ * @pattern: FcPattern to examine
  *
  * The supplied FcPattern must supply file and family information,
  * otherwise this function will fail.
@@ -447,7 +528,10 @@ font_manager_get_langs_from_fontconfig_pattern (FcPattern *pattern)
  * character set and spacing information, however default values will be used
  * if those fields are missing. All other fields are ignored.
  *
- * Returns: (transfer full): a newly created #JsonObject
+ * See #FontManagerFont for a description of the #JsonObject returned by this function.
+ *
+ * Returns: (transfer full): A newly created #JsonObject which should be
+ * freed using #json_object_unref() when no longer needed.
  */
 JsonObject *
 font_manager_get_attributes_from_fontconfig_pattern (FcPattern *pattern)
@@ -517,41 +601,42 @@ font_manager_get_attributes_from_fontconfig_pattern (FcPattern *pattern)
 
 /**
  * font_manager_get_attributes_from_filepath:
- * @filepath: full path to font file to query
- * @index: index of face within file to select
+ * @filepath:   full path to font file to query
+ * @index:      index of face within file to select
+ * @error:      #GError or %NULL to ignore errors
  *
- * If an error is encontered, the returned object will have a member named err
- * set to %TRUE and a member named err_msg containing a description of the error.
+ * See #FontManagerFont for a description of the #JsonObject returned by this function.
  *
- * Returns: (transfer full): a newly created #JsonObject
+ * Returns: (transfer full): A newly created #JsonObject which should be
+ * freed using #json_object_unref() when no longer needed or %NULL if there was an error.
  */
 JsonObject *
-font_manager_get_attributes_from_filepath (const gchar *filepath, int index)
+font_manager_get_attributes_from_filepath (const gchar *filepath, int index, GError **error)
 {
+    g_return_val_if_fail(filepath != NULL, NULL);
+    g_return_val_if_fail(index >= 0, NULL);
+    g_return_val_if_fail((error == NULL || *error == NULL), NULL);
+
     int count;
     FcBlanks *blanks = FcBlanksCreate();
     FcPattern *pattern = FcFreeTypeQuery((const FcChar8 *) filepath, index, blanks, &count);
 
     JsonObject *json_obj = NULL;
 
-    if (pattern) {
+    if (pattern)
         json_obj = font_manager_get_attributes_from_fontconfig_pattern(pattern);
-    } else {
-        json_obj = json_object_new();
-        font_manager_set_json_error(json_obj, 0, "Failed to create FontConfig pattern for file");
-        if (blanks)
-            FcBlanksDestroy(blanks);
-        return json_obj;
-    }
+    else
+        set_error("Failed to create FontConfig pattern for file", error);
 
     FcBlanksDestroy(blanks);
-    FcPatternDestroy(pattern);
+    if (pattern)
+        FcPatternDestroy(pattern);
     return json_obj;
 }
 
 /**
  * font_manager_get_charset_from_fontconfig_pattern: (skip)
- * @pattern: #FcPattern to examine
+ * @pattern: FcPattern to examine
  *
  * Supplied FcPattern must contain an FcCharSet.
  *
@@ -648,7 +733,32 @@ font_manager_get_charset_from_filepath (const gchar *filepath, int index)
  * font_manager_sort_json_font_listing:
  * @json_obj: #JsonObject returned from #font_manager_get_available_fonts*
  *
- * Returns : (transfer full) : #JsonArray
+ * The returned #JsonArray has the following structure:
+ *|[
+ * [
+ *   {
+ *     "family" : string,
+ *     "n_variations" : int,
+ *     "variations" : [
+ *       {
+ *         "filepath" : string,
+ *         "findex" : int,
+ *         "family" : string,
+ *         "style" : string,
+ *         "spacing" : int,
+ *         "slant" : int,
+ *         "weight" : int,
+ *         "width" : int,
+ *         "description" : string,
+ *       },
+ *       ...
+ *     ],
+ *     "description" : string,
+ *   },
+ *   ...
+ * ]
+ *]|
+ * Returns: (transfer full): #JsonArray
  */
 JsonArray *
 font_manager_sort_json_font_listing (JsonObject *json_obj)
@@ -679,7 +789,7 @@ font_manager_sort_json_font_listing (JsonObject *json_obj)
             if (!json_object_get_member(_family_obj, "description")) {
                 const gchar *style = json_object_get_string_member(style_obj, "style");
                 for (guint i = 0; i < G_N_ELEMENTS(DEFAULT_VARIANTS); i++) {
-                    if (g_strrstr(style, DEFAULT_VARIANTS[i]) != NULL) {
+                    if (g_strcmp0(style, DEFAULT_VARIANTS[i]) == 0) {
                         const gchar *font_desc = json_object_get_string_member(style_obj, "description");
                         json_object_set_string_member(_family_obj, "description", font_desc);
                         break;
@@ -702,51 +812,400 @@ font_manager_sort_json_font_listing (JsonObject *json_obj)
     return result;
 }
 
-/* From pangofc-fontmap.c */
-static GList *
-list_charset (const FcCharSet *charset)
+/**
+ * font_manager_weight_to_string:
+ * @weight: #FontManagerWeight
+ *
+ * Returns: (transfer none) (nullable): @weight as a string
+ */
+const gchar *
+font_manager_weight_to_string (FontManagerWeight weight)
 {
-    GList *result = NULL;
-    FcChar32  ucs4, pos;
-    FcChar32  map[FC_CHARSET_MAP_SIZE];
-
-    for (ucs4 = FcCharSetFirstPage (charset, map, &pos);
-         ucs4 != FC_CHARSET_DONE;
-         ucs4 = FcCharSetNextPage (charset, map, &pos)) {
-
-        for (int i = 0; i < FC_CHARSET_MAP_SIZE; i++) {
-            int b = 0;
-            FcChar32 bits = map[i];
-            FcChar32 base = ucs4 + i * 32;
-            while (bits) {
-                if (bits & 1) {
-                    gunichar ch = (base + b);
-                    if (unicode_unichar_isgraph(ch))
-                        result = g_list_prepend(result, GINT_TO_POINTER(ch));
-                }
-                bits >>= 1;
-                b++;
-            }
-        }
-
+    switch (weight) {
+        case FONT_MANAGER_WEIGHT_THIN:
+            return _("Thin");
+        case FONT_MANAGER_WEIGHT_ULTRALIGHT:
+            return _("Ultra-Light");
+        case FONT_MANAGER_WEIGHT_LIGHT:
+            return _("Light");
+        case FONT_MANAGER_WEIGHT_SEMILIGHT:
+            return _("Semi-Light");
+        case FONT_MANAGER_WEIGHT_BOOK:
+            return _("Book");
+        case FONT_MANAGER_WEIGHT_MEDIUM:
+            return _("Medium");
+        case FONT_MANAGER_WEIGHT_SEMIBOLD:
+            return _("Semi-Bold");
+        case FONT_MANAGER_WEIGHT_BOLD:
+            return _("Bold");
+        case FONT_MANAGER_WEIGHT_ULTRABOLD:
+            return _("Ultra-Bold");
+        case FONT_MANAGER_WEIGHT_HEAVY:
+            return _("Heavy");
+        case FONT_MANAGER_WEIGHT_ULTRABLACK:
+            return _("Ultra-Heavy");
+        default:
+            return NULL;
     }
-
-    return g_list_reverse(result);
 }
 
-static void
-process_fontset (const FcFontSet *fontset, JsonObject *json_obj)
+/**
+ * font_manager_weight_defined:
+ * @weight:     #FontManagerWeight
+ *
+ * Returns:     %TRUE if @weight is valid.
+ */
+gboolean
+font_manager_weight_defined (FontManagerWeight weight)
 {
-    for (int i = 0; i < fontset->nfont; i++) {
-        if (pango_version() >= PANGO_1_44 && is_legacy_format(fontset->fonts[i]))
-            continue;
-        JsonObject *font_obj = font_manager_get_attributes_from_fontconfig_pattern(fontset->fonts[i]);
-        const gchar *family = json_object_get_string_member(font_obj, "family");
-        const gchar *style = json_object_get_string_member(font_obj, "style");
-        if (!json_object_get_member(json_obj, family))
-            json_object_set_object_member(json_obj, family, json_object_new());
-        JsonObject *family_obj = json_object_get_object_member(json_obj, family);
-        json_object_set_object_member(family_obj, style, font_obj);
+    switch (weight) {
+        case FONT_MANAGER_WEIGHT_THIN:
+        case FONT_MANAGER_WEIGHT_ULTRALIGHT:
+        case FONT_MANAGER_WEIGHT_LIGHT:
+        case FONT_MANAGER_WEIGHT_SEMILIGHT:
+        case FONT_MANAGER_WEIGHT_BOOK:
+        case FONT_MANAGER_WEIGHT_REGULAR:
+        case FONT_MANAGER_WEIGHT_MEDIUM:
+        case FONT_MANAGER_WEIGHT_SEMIBOLD:
+        case FONT_MANAGER_WEIGHT_BOLD:
+        case FONT_MANAGER_WEIGHT_ULTRABOLD:
+        case FONT_MANAGER_WEIGHT_HEAVY:
+        case FONT_MANAGER_WEIGHT_ULTRABLACK:
+            return TRUE;
+        default:
+            return FALSE;
     }
-    return;
+}
+
+GType
+font_manager_weight_get_type (void)
+{
+  static volatile gsize g_define_type_id__volatile = 0;
+
+  if (g_once_init_enter (&g_define_type_id__volatile))
+    {
+      static const GEnumValue values[] = {
+        { FONT_MANAGER_WEIGHT_THIN, "FONT_MANAGER_WEIGHT_THIN", "thin" },
+        { FONT_MANAGER_WEIGHT_ULTRALIGHT, "FONT_MANAGER_WEIGHT_ULTRALIGHT", "ultralight" },
+        { FONT_MANAGER_WEIGHT_LIGHT, "FONT_MANAGER_WEIGHT_LIGHT", "light" },
+        { FONT_MANAGER_WEIGHT_SEMILIGHT, "FONT_MANAGER_WEIGHT_SEMILIGHT", "semilight" },
+        { FONT_MANAGER_WEIGHT_BOOK, "FONT_MANAGER_WEIGHT_BOOK", "book" },
+        { FONT_MANAGER_WEIGHT_REGULAR, "FONT_MANAGER_WEIGHT_REGULAR", "regular" },
+        { FONT_MANAGER_WEIGHT_MEDIUM, "FONT_MANAGER_WEIGHT_MEDIUM", "medium" },
+        { FONT_MANAGER_WEIGHT_SEMIBOLD, "FONT_MANAGER_WEIGHT_SEMIBOLD", "semibold" },
+        { FONT_MANAGER_WEIGHT_BOLD, "FONT_MANAGER_WEIGHT_BOLD", "bold" },
+        { FONT_MANAGER_WEIGHT_ULTRABOLD, "FONT_MANAGER_WEIGHT_ULTRABOLD", "ultrabold" },
+        { FONT_MANAGER_WEIGHT_HEAVY, "FONT_MANAGER_WEIGHT_HEAVY", "heavy" },
+        { FONT_MANAGER_WEIGHT_ULTRABLACK, "FONT_MANAGER_WEIGHT_ULTRABLACK", "ultrablack" },
+        { 0, NULL, NULL }
+      };
+      GType g_define_type_id =
+        g_enum_register_static (g_intern_static_string ("FontManagerWeight"), values);
+      g_once_init_leave (&g_define_type_id__volatile, g_define_type_id);
+    }
+
+  return g_define_type_id__volatile;
+}
+
+/**
+ * font_manager_slant_to_string:
+ * @slant:  #FontManagerSlant
+ *
+ * Returns: (transfer none) (nullable): @slant as a string
+ */
+const gchar *
+font_manager_slant_to_string (FontManagerSlant slant)
+{
+    switch (slant) {
+        case FONT_MANAGER_SLANT_ITALIC:
+            return _("Italic");
+        case FONT_MANAGER_SLANT_OBLIQUE:
+            return _("Oblique");
+        default:
+            return NULL;
+    }
+}
+
+GType
+font_manager_slant_get_type (void)
+{
+  static volatile gsize g_define_type_id__volatile = 0;
+
+  if (g_once_init_enter (&g_define_type_id__volatile))
+    {
+      static const GEnumValue values[] = {
+        { FONT_MANAGER_SLANT_ROMAN, "FONT_MANAGER_SLANT_ROMAN", "roman" },
+        { FONT_MANAGER_SLANT_ITALIC, "FONT_MANAGER_SLANT_ITALIC", "italic" },
+        { FONT_MANAGER_SLANT_OBLIQUE, "FONT_MANAGER_SLANT_OBLIQUE", "oblique" },
+        { 0, NULL, NULL }
+      };
+      GType g_define_type_id =
+        g_enum_register_static (g_intern_static_string ("FontManagerSlant"), values);
+      g_once_init_leave (&g_define_type_id__volatile, g_define_type_id);
+    }
+
+  return g_define_type_id__volatile;
+}
+
+/**
+ * font_manager_width_to_string:
+ * @width:  #FontManagerWidth
+ *
+ * Returns: (transfer none) (nullable): @width as a string
+ */
+const gchar *
+font_manager_width_to_string (FontManagerWidth width)
+{
+    switch (width) {
+        case FONT_MANAGER_WIDTH_ULTRACONDENSED:
+            return _("Ultra-Condensed");
+        case FONT_MANAGER_WIDTH_EXTRACONDENSED:
+            return _("Extra-Condensed");
+        case FONT_MANAGER_WIDTH_CONDENSED:
+            return _("Condensed");
+        case FONT_MANAGER_WIDTH_SEMICONDENSED:
+            return _("Semi-Condensed");
+        case FONT_MANAGER_WIDTH_SEMIEXPANDED:
+            return _("Semi-Expanded");
+        case FONT_MANAGER_WIDTH_EXPANDED:
+            return _("Expanded");
+        case FONT_MANAGER_WIDTH_EXTRAEXPANDED:
+            return _("Extra-Expanded");
+        case FONT_MANAGER_WIDTH_ULTRAEXPANDED:
+            return _("Ultra-Expanded");
+        default:
+            return NULL;
+    }
+}
+
+/**
+ * font_manager_width_defined:
+ * @width:     #FontManagerWidth
+ *
+ * Returns:     %TRUE if @weight is valid.
+ */
+gboolean
+font_manager_width_defined (FontManagerWidth width)
+{
+    switch (width) {
+        case FONT_MANAGER_WIDTH_NORMAL:
+        case FONT_MANAGER_WIDTH_ULTRACONDENSED:
+        case FONT_MANAGER_WIDTH_EXTRACONDENSED:
+        case FONT_MANAGER_WIDTH_CONDENSED:
+        case FONT_MANAGER_WIDTH_SEMICONDENSED:
+        case FONT_MANAGER_WIDTH_SEMIEXPANDED:
+        case FONT_MANAGER_WIDTH_EXPANDED:
+        case FONT_MANAGER_WIDTH_EXTRAEXPANDED:
+        case FONT_MANAGER_WIDTH_ULTRAEXPANDED:
+            return TRUE;
+        default:
+            return FALSE;
+    }
+}
+
+GType
+font_manager_width_get_type (void)
+{
+  static volatile gsize g_define_type_id__volatile = 0;
+
+  if (g_once_init_enter (&g_define_type_id__volatile))
+    {
+      static const GEnumValue values[] = {
+        { FONT_MANAGER_WIDTH_ULTRACONDENSED, "FONT_MANAGER_WIDTH_ULTRACONDENSED", "ultracondensed" },
+        { FONT_MANAGER_WIDTH_EXTRACONDENSED, "FONT_MANAGER_WIDTH_EXTRACONDENSED", "extracondensed" },
+        { FONT_MANAGER_WIDTH_CONDENSED, "FONT_MANAGER_WIDTH_CONDENSED", "condensed" },
+        { FONT_MANAGER_WIDTH_SEMICONDENSED, "FONT_MANAGER_WIDTH_SEMICONDENSED", "semicondensed" },
+        { FONT_MANAGER_WIDTH_NORMAL, "FONT_MANAGER_WIDTH_NORMAL", "normal" },
+        { FONT_MANAGER_WIDTH_SEMIEXPANDED, "FONT_MANAGER_WIDTH_SEMIEXPANDED", "semiexpanded" },
+        { FONT_MANAGER_WIDTH_EXPANDED, "FONT_MANAGER_WIDTH_EXPANDED", "expanded" },
+        { FONT_MANAGER_WIDTH_EXTRAEXPANDED, "FONT_MANAGER_WIDTH_EXTRAEXPANDED", "extraexpanded" },
+        { FONT_MANAGER_WIDTH_ULTRAEXPANDED, "FONT_MANAGER_WIDTH_ULTRAEXPANDED", "ultraexpanded" },
+        { 0, NULL, NULL }
+      };
+      GType g_define_type_id =
+        g_enum_register_static (g_intern_static_string ("FontManagerWidth"), values);
+      g_once_init_leave (&g_define_type_id__volatile, g_define_type_id);
+    }
+
+  return g_define_type_id__volatile;
+}
+
+/**
+ * font_manager_spacing_to_string:
+ * @spacing:    #FontManagerSpacing
+ *
+ * Returns: (transfer none) (nullable): @spacing as a string
+ */
+const gchar *
+font_manager_spacing_to_string (FontManagerSpacing spacing)
+{
+    switch (spacing) {
+        case FONT_MANAGER_SPACING_PROPORTIONAL:
+            return _("Proportional");
+        case FONT_MANAGER_SPACING_DUAL:
+            return _("Dual Width");
+        case FONT_MANAGER_SPACING_MONO:
+            return _("Monospace");
+        case FONT_MANAGER_SPACING_CHARCELL:
+            return _("Charcell");
+        default:
+            return NULL;
+    }
+}
+
+GType
+font_manager_spacing_get_type (void)
+{
+  static volatile gsize g_define_type_id__volatile = 0;
+
+  if (g_once_init_enter (&g_define_type_id__volatile))
+    {
+      static const GEnumValue values[] = {
+        { FONT_MANAGER_SPACING_PROPORTIONAL, "FONT_MANAGER_SPACING_PROPORTIONAL", "proportional" },
+        { FONT_MANAGER_SPACING_DUAL, "FONT_MANAGER_SPACING_DUAL", "dual" },
+        { FONT_MANAGER_SPACING_MONO, "FONT_MANAGER_SPACING_MONO", "mono" },
+        { FONT_MANAGER_SPACING_CHARCELL, "FONT_MANAGER_SPACING_CHARCELL", "charcell" },
+        { 0, NULL, NULL }
+      };
+      GType g_define_type_id =
+        g_enum_register_static (g_intern_static_string ("FontManagerSpacing"), values);
+      g_once_init_leave (&g_define_type_id__volatile, g_define_type_id);
+    }
+
+  return g_define_type_id__volatile;
+}
+
+/**
+ * font_manager_subpixel_order_to_string:
+ * @rgba:   #FontManagerSubpixelOrder
+ *
+ * Returns: (transfer none) (nullable): @rgba as a string
+ */
+const gchar *
+font_manager_subpixel_order_to_string (FontManagerSubpixelOrder rgba)
+{
+    switch (rgba) {
+        case FONT_MANAGER_SUBPIXEL_ORDER_UNKNOWN:
+            return _("Unknown");
+        case FONT_MANAGER_SUBPIXEL_ORDER_RGB:
+            return _("RGB");
+        case FONT_MANAGER_SUBPIXEL_ORDER_BGR:
+            return _("BGR");
+        case FONT_MANAGER_SUBPIXEL_ORDER_VRGB:
+            return _("VRGB");
+        case FONT_MANAGER_SUBPIXEL_ORDER_VBGR:
+            return _("VBGR");
+        default:
+            return _("None");
+    }
+}
+
+GType
+font_manager_subpixel_order_get_type (void)
+{
+  static volatile gsize g_define_type_id__volatile = 0;
+
+  if (g_once_init_enter (&g_define_type_id__volatile))
+    {
+      static const GEnumValue values[] = {
+        { FONT_MANAGER_SUBPIXEL_ORDER_UNKNOWN, "FONT_MANAGER_SUBPIXEL_ORDER_UNKNOWN", "unknown" },
+        { FONT_MANAGER_SUBPIXEL_ORDER_RGB, "FONT_MANAGER_SUBPIXEL_ORDER_RGB", "rgb" },
+        { FONT_MANAGER_SUBPIXEL_ORDER_BGR, "FONT_MANAGER_SUBPIXEL_ORDER_BGR", "bgr" },
+        { FONT_MANAGER_SUBPIXEL_ORDER_VRGB, "FONT_MANAGER_SUBPIXEL_ORDER_VRGB", "vrgb" },
+        { FONT_MANAGER_SUBPIXEL_ORDER_VBGR, "FONT_MANAGER_SUBPIXEL_ORDER_VBGR", "vbgr" },
+        { FONT_MANAGER_SUBPIXEL_ORDER_NONE, "FONT_MANAGER_SUBPIXEL_ORDER_NONE", "none" },
+        { 0, NULL, NULL }
+      };
+      GType g_define_type_id =
+        g_enum_register_static (g_intern_static_string ("FontManagerSubpixelOrder"), values);
+      g_once_init_leave (&g_define_type_id__volatile, g_define_type_id);
+    }
+
+  return g_define_type_id__volatile;
+}
+
+/**
+ * font_manager_hint_style_to_string:
+ * @hinting:    #FontManagerHintStyle
+ *
+ * Returns: (transfer none) (nullable): @hinting as a string
+ */
+const gchar *
+font_manager_hint_style_to_string (FontManagerHintStyle hinting)
+{
+    switch (hinting) {
+        case FONT_MANAGER_HINT_STYLE_SLIGHT:
+            return _("Slight");
+        case FONT_MANAGER_HINT_STYLE_MEDIUM:
+            return _("Medium");
+        case FONT_MANAGER_HINT_STYLE_FULL:
+            return _("Full");
+        default:
+            return _("None");
+    }
+}
+
+GType
+font_manager_hint_style_get_type (void)
+{
+  static volatile gsize g_define_type_id__volatile = 0;
+
+  if (g_once_init_enter (&g_define_type_id__volatile))
+    {
+      static const GEnumValue values[] = {
+        { FONT_MANAGER_HINT_STYLE_NONE, "FONT_MANAGER_HINT_STYLE_NONE", "none" },
+        { FONT_MANAGER_HINT_STYLE_SLIGHT, "FONT_MANAGER_HINT_STYLE_SLIGHT", "slight" },
+        { FONT_MANAGER_HINT_STYLE_MEDIUM, "FONT_MANAGER_HINT_STYLE_MEDIUM", "medium" },
+        { FONT_MANAGER_HINT_STYLE_FULL, "FONT_MANAGER_HINT_STYLE_FULL", "full" },
+        { 0, NULL, NULL }
+      };
+      GType g_define_type_id =
+        g_enum_register_static (g_intern_static_string ("FontManagerHintStyle"), values);
+      g_once_init_leave (&g_define_type_id__volatile, g_define_type_id);
+    }
+
+  return g_define_type_id__volatile;
+}
+
+/**
+ * font_manager_lcd_filter_to_string:
+ * @filter: #FontManagerLCDFilter
+ *
+ * Returns: (transfer none) (nullable): @filter as a string
+ */
+const gchar *
+font_manager_lcd_filter_to_string (FontManagerLCDFilter filter)
+{
+    switch (filter) {
+        case FONT_MANAGER_LCD_FILTER_DEFAULT:
+            return _("Default");
+        case FONT_MANAGER_LCD_FILTER_LIGHT:
+            return _("Light");
+        case FONT_MANAGER_LCD_FILTER_LEGACY:
+            return _("Legacy");
+        default:
+            return _("None");
+    }
+}
+
+GType
+font_manager_lcd_filter_get_type (void)
+{
+  static volatile gsize g_define_type_id__volatile = 0;
+
+  if (g_once_init_enter (&g_define_type_id__volatile))
+    {
+      static const GEnumValue values[] = {
+        { FONT_MANAGER_LCD_FILTER_NONE, "FONT_MANAGER_LCD_FILTER_NONE", "none" },
+        { FONT_MANAGER_LCD_FILTER_DEFAULT, "FONT_MANAGER_LCD_FILTER_DEFAULT", "default" },
+        { FONT_MANAGER_LCD_FILTER_LIGHT, "FONT_MANAGER_LCD_FILTER_LIGHT", "light" },
+        { FONT_MANAGER_LCD_FILTER_LEGACY, "FONT_MANAGER_LCD_FILTER_LEGACY", "legacy" },
+        { 0, NULL, NULL }
+      };
+      GType g_define_type_id =
+        g_enum_register_static (g_intern_static_string ("FontManagerLCDFilter"), values);
+      g_once_init_leave (&g_define_type_id__volatile, g_define_type_id);
+    }
+
+  return g_define_type_id__volatile;
 }
