@@ -1,6 +1,6 @@
 /* font-manager-properties.c
  *
- * Copyright (C) 2009 - 2020 Jerry Casiano
+ * Copyright (C) 2009 - 2021 Jerry Casiano
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,13 +18,18 @@
  * If not, see <http://www.gnu.org/licenses/gpl-3.0.txt>.
 */
 
-#include <locale.h>
-#include <stdlib.h>
-#include <gio/gio.h>
-#include <fontconfig/fontconfig.h>
-
 #include "font-manager-properties.h"
 
+/**
+ * SECTION: font-manager-properties
+ * @short_description: Fontconfig property configuration
+ * @title: Font Properties
+ * @include: font-manager-properties.h
+ *
+ * Base class for generating fontconfig configuration files which modify
+ * font properties.
+ */
+
 typedef struct
 {
     gchar *config_dir;
@@ -72,30 +77,6 @@ static GParamSpec *obj_properties[N_PROPERTIES] = {0};
 
 #define DEFAULT_PARAM_FLAGS (G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS)
 
-static const struct
-{
-    const gchar *name;
-    GType type;
-}
-PROPERTIES [] =
-{
-    { "RESERVED", G_TYPE_RESERVED_GLIB_FIRST },
-    { "hintstyle", G_TYPE_INT },
-    { "antialias", G_TYPE_BOOLEAN },
-    { "hinting", G_TYPE_BOOLEAN },
-    { "autohint", G_TYPE_BOOLEAN },
-    { "embeddedbitmap", G_TYPE_BOOLEAN },
-    { "less", G_TYPE_DOUBLE },
-    { "more", G_TYPE_DOUBLE },
-    { "rgba", G_TYPE_INT },
-    { "lcdfilter", G_TYPE_INT },
-    { "scale", G_TYPE_DOUBLE },
-    { "dpi", G_TYPE_DOUBLE },
-    { "config-dir", G_TYPE_STRING },
-    { "target-file", G_TYPE_STRING },
-    { "type", G_TYPE_INT }
-};
-
 static const struct
 {
     gint start;
@@ -107,15 +88,35 @@ PROPERTY_ID_RANGE [] =
     { 8, 11 }
 };
 
-static void
-font_manager_properties_finalize (GObject *self)
+GType
+font_manager_properties_type_get_type (void)
 {
-    FontManagerProperties *_self = FONT_MANAGER_PROPERTIES(self);
-    g_return_if_fail(_self != NULL);
-    FontManagerPropertiesPrivate *priv = font_manager_properties_get_instance_private(_self);
-    g_free(priv->config_dir);
-    g_free(priv->target_file);
-    G_OBJECT_CLASS(font_manager_properties_parent_class)->finalize(self);
+  static volatile gsize g_define_type_id__volatile = 0;
+
+  if (g_once_init_enter (&g_define_type_id__volatile))
+    {
+      static const GEnumValue values[] = {
+        { FONT_MANAGER_PROPERTIES_TYPE_DEFAULT, "FONT_MANAGER_PROPERTIES_TYPE_DEFAULT", "default" },
+        { FONT_MANAGER_PROPERTIES_TYPE_DISPLAY, "FONT_MANAGER_PROPERTIES_TYPE_DISPLAY", "display" },
+        { 0, NULL, NULL }
+      };
+      GType g_define_type_id =
+        g_enum_register_static (g_intern_static_string ("FontManagerPropertiesType"), values);
+      g_once_init_leave (&g_define_type_id__volatile, g_define_type_id);
+    }
+
+  return g_define_type_id__volatile;
+}
+
+static void
+font_manager_properties_dispose (GObject *gobject)
+{
+    g_return_if_fail(gobject != NULL);
+    FontManagerProperties *self = FONT_MANAGER_PROPERTIES(gobject);
+    FontManagerPropertiesPrivate *priv = font_manager_properties_get_instance_private(self);
+    g_clear_pointer(&priv->config_dir, g_free);
+    g_clear_pointer(&priv->target_file, g_free);
+    G_OBJECT_CLASS(font_manager_properties_parent_class)->dispose(gobject);
 }
 
 static void
@@ -124,8 +125,8 @@ font_manager_properties_get_property (GObject *gobject,
                                      GValue *value,
                                      GParamSpec *pspec)
 {
+    g_return_if_fail(gobject != NULL);
     FontManagerProperties *self = FONT_MANAGER_PROPERTIES(gobject);
-    g_return_if_fail(self != NULL);
     FontManagerPropertiesPrivate *priv = font_manager_properties_get_instance_private(self);
     switch (property_id) {
         case PROP_HINTSTYLE:
@@ -183,8 +184,8 @@ font_manager_properties_set_property (GObject *gobject,
                                      const GValue *value,
                                      GParamSpec *pspec)
 {
+    g_return_if_fail(gobject != NULL);
     FontManagerProperties *self = FONT_MANAGER_PROPERTIES(gobject);
-    g_return_if_fail(self != NULL);
     FontManagerPropertiesPrivate *priv = font_manager_properties_get_instance_private(self);
     switch (property_id) {
         case PROP_HINTSTYLE:
@@ -341,6 +342,7 @@ font_manager_properties_add_assignments (FontManagerProperties *self,
         /* Skip test elements, handled in add_match_criteria */
         if (i == PROP_LESS || i == PROP_MORE)
             continue;
+        g_autofree gchar *loc = NULL;
         g_autofree gchar *val = NULL;
         g_autofree gchar *val_type = NULL;
         const gchar *name = PROPERTIES[i].name;
@@ -356,12 +358,11 @@ font_manager_properties_add_assignments (FontManagerProperties *self,
                 break;
             }
             case G_TYPE_DOUBLE: {
-                ; /* Empty statement */
-                g_autofree gchar *locale = g_strdup(setlocale(LC_ALL, NULL));
+                loc = g_strdup(setlocale(LC_ALL, NULL));
                 setlocale(LC_ALL, "C");
                 val = g_strdup_printf("%.1f", g_value_get_double(&value));
                 val_type = g_strdup("double");
-                setlocale(LC_ALL, locale);
+                setlocale(LC_ALL, loc);
                 break;
             }
             case G_TYPE_BOOLEAN: {
@@ -403,7 +404,7 @@ static void
 font_manager_properties_class_init (FontManagerPropertiesClass *klass)
 {
     GObjectClass *object_class = G_OBJECT_CLASS(klass);
-    object_class->finalize = font_manager_properties_finalize;
+    object_class->dispose = font_manager_properties_dispose;
     object_class->get_property = font_manager_properties_get_property;
     object_class->set_property = font_manager_properties_set_property;
 
@@ -416,28 +417,34 @@ font_manager_properties_class_init (FontManagerPropertiesClass *klass)
         switch (PROPERTIES[i].type) {
             case G_TYPE_INT:
                 obj_properties[i] = g_param_spec_int(PROPERTIES[i].name,
-                                                    NULL, NULL,
+                                                    NULL,
+                                                    PROPERTIES[i].desc,
                                                     0,
-                                                    i == PROP_LCDFILTER ? 6 : 4,
+                                                    i == PROP_LCDFILTER ? 6 :
+                                                    i == PROP_TYPE ? 1 : 4,
                                                     0,
                                                     DEFAULT_PARAM_FLAGS);
                 break;
             case G_TYPE_BOOLEAN:
                 obj_properties[i] = g_param_spec_boolean(PROPERTIES[i].name,
-                                                         NULL, NULL,
+                                                         NULL,
+                                                         PROPERTIES[i].desc,
                                                          FALSE,
                                                          DEFAULT_PARAM_FLAGS);
                 break;
             case G_TYPE_DOUBLE:
                 obj_properties[i] = g_param_spec_double(PROPERTIES[i].name,
-                                                        NULL, NULL,
+                                                        NULL,
+                                                        PROPERTIES[i].desc,
                                                         0.0, G_MAXDOUBLE,
                                                         get_default_for_double_property(i),
                                                         DEFAULT_PARAM_FLAGS);
                 break;
             case G_TYPE_STRING:
                 obj_properties[i] = g_param_spec_string(PROPERTIES[i].name,
-                                                       NULL, NULL, NULL,
+                                                       NULL,
+                                                       PROPERTIES[i].desc,
+                                                       NULL,
                                                        DEFAULT_PARAM_FLAGS);
                 break;
             case G_TYPE_RESERVED_GLIB_FIRST:
@@ -465,7 +472,7 @@ font_manager_properties_init (FontManagerProperties *self)
 
 /**
  * font_manager_properties_load:
- * @self    #FontManagerProperties
+ * @self:   #FontManagerProperties
  *
  * Returns: %TRUE if current configuration file was successfully loaded
  */
@@ -481,12 +488,10 @@ font_manager_properties_load (FontManagerProperties *self)
     if (!g_file_query_exists(file, NULL))
         return FALSE;
 
-    xmlInitParser();
     xmlDoc *doc = xmlReadFile(filepath, NULL, 0);
 
     if (doc == NULL) {
         /* Empty file */
-        xmlCleanupParser();
         return FALSE;
     }
 
@@ -507,13 +512,12 @@ font_manager_properties_load (FontManagerProperties *self)
     }
 
     xmlFreeDoc(doc);
-    xmlCleanupParser();
     return TRUE;
 }
 
 /**
  * font_manager_properties_save:
- * @self    #FontManagerProperties
+ * @self:   #FontManagerProperties
  *
  * Returns: %TRUE if current configuration was successfully saved to file
  */
@@ -572,6 +576,7 @@ font_manager_properties_get_filepath (FontManagerProperties *self)
 
 /**
  * font_manager_properties_reset:
+ * @self:   #FontManagerProperties
  *
  * Reset all base properties to their default values
  */
@@ -660,8 +665,8 @@ font_manager_properties_reset (FontManagerProperties *self)
 /**
  * font_manager_properties_new:
  *
- * Returns: (transfer full): #FontManagerProperties
- * Use #g_object_unref() to free result.
+ * Returns: (transfer full): A newly created #FontManagerProperties
+ * Free the returned object using #g_object_unref().
  */
 FontManagerProperties *
 font_manager_properties_new (void)
