@@ -2,7 +2,7 @@
  *
  * Originally a part of Gucharmap
  *
- * Copyright (C) 2017 - 2020 Jerry Casiano
+ * Copyright (C) 2017 - 2021 Jerry Casiano
  *
  *
  * Copyright © 2004 Noah Levitt
@@ -36,6 +36,15 @@
 #include "unicode-character-map.h"
 #include "unicode-character-map-zoom-window.h"
 
+/**
+ * SECTION: unicode-character-map
+ * @short_description: Character map widget
+ * @title: Character Map
+ * @include: unicode-character-map.h
+ *
+ * Widget which displays all the available characters in the selected font.
+ */
+
 #define VALID_FONT_SIZE(X) (X < 6.0 ? 6.0 : X > 96.0 ? 96.0 : X)
 
 /* Notes
@@ -97,7 +106,7 @@ G_DEFINE_TYPE_WITH_CODE (UnicodeCharacterMap, unicode_character_map,
                          G_IMPLEMENT_INTERFACE(GTK_TYPE_SCROLLABLE, NULL)
                          G_ADD_PRIVATE(UnicodeCharacterMap))
 
-/* Set during initial construction */
+/* XXX: FIXME! Set during initial construction - this IS a bad idea */
 UnicodeCharacterMapPrivate *priv = NULL;
 
 /* These are chosen for compatibility with the older code that didn't
@@ -156,10 +165,7 @@ static void
 unicode_character_map_clear_pango_layout (UnicodeCharacterMap *charmap)
 {
     g_return_if_fail(charmap != NULL);
-    if (priv->pango_layout) {
-        g_object_unref(priv->pango_layout);
-        priv->pango_layout = NULL;
-    }
+    g_clear_object(&priv->pango_layout);
     return;
 }
 
@@ -927,8 +933,7 @@ unicode_character_map_set_vadjustment (UnicodeCharacterMap *charmap, GtkAdjustme
     if (priv->vadjustment) {
         g_signal_handler_disconnect(priv->vadjustment, priv->vadjustment_changed_handler_id);
         priv->vadjustment_changed_handler_id = 0;
-        g_object_unref(priv->vadjustment);
-        priv->vadjustment = NULL;
+        g_clear_object(&priv->vadjustment);
     }
 
     if (vadjustment) {
@@ -1148,36 +1153,28 @@ unicode_character_map_init (UnicodeCharacterMap *charmap)
 }
 
 static void
-unicode_character_map_finalize (GObject *object)
+unicode_character_map_dispose (GObject *gobject)
 {
-    UnicodeCharacterMap *charmap = UNICODE_CHARACTER_MAP(object);
-
-    if (priv->font_desc)
-        pango_font_description_free(priv->font_desc);
-
+    g_return_if_fail(gobject != NULL);
+    UnicodeCharacterMap *charmap = UNICODE_CHARACTER_MAP(gobject);
+    g_clear_pointer(&priv->font_desc, pango_font_description_free);
     unicode_character_map_clear_pango_layout(charmap);
-    gtk_target_list_unref(priv->target_list);
-
-    if (priv->codepoint_list)
-        g_object_unref(priv->codepoint_list);
-
-    if (priv->long_press)
-        g_object_unref(priv->long_press);
-
-    if (priv->pinch_zoom)
-        g_object_unref(priv->pinch_zoom);
-
-    G_OBJECT_CLASS(unicode_character_map_parent_class)->finalize(object);
+    g_clear_pointer(&priv->target_list, gtk_target_list_unref);
+    g_clear_object(&priv->codepoint_list);
+    g_clear_object(&priv->long_press);
+    g_clear_object(&priv->pinch_zoom);
+    G_OBJECT_CLASS(unicode_character_map_parent_class)->dispose(gobject);
     return;
 }
 
 static void
-unicode_character_map_set_property (GObject *object,
+unicode_character_map_set_property (GObject *gobject,
                                     guint prop_id,
                                     const GValue *value,
                                     GParamSpec *pspec)
 {
-    UnicodeCharacterMap *charmap = UNICODE_CHARACTER_MAP(object);
+    g_return_if_fail(gobject != NULL);
+    UnicodeCharacterMap *charmap = UNICODE_CHARACTER_MAP(gobject);
 
     switch (prop_id) {
         case PROP_HADJUSTMENT:
@@ -1207,19 +1204,20 @@ unicode_character_map_set_property (GObject *object,
             unicode_character_map_set_preview_size(charmap, g_value_get_double(value));
             break;
         default:
-            G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
+            G_OBJECT_WARN_INVALID_PROPERTY_ID(gobject, prop_id, pspec);
             break;
     }
     return;
 }
 
 static void
-unicode_character_map_get_property (GObject *object,
+unicode_character_map_get_property (GObject *gobject,
                                     guint prop_id,
                                     GValue *value,
                                     GParamSpec *pspec)
 {
-    UnicodeCharacterMap *charmap = UNICODE_CHARACTER_MAP(object);
+    g_return_if_fail(gobject != NULL);
+    UnicodeCharacterMap *charmap = UNICODE_CHARACTER_MAP(gobject);
 
     switch (prop_id) {
         case PROP_HADJUSTMENT:
@@ -1247,7 +1245,7 @@ unicode_character_map_get_property (GObject *object,
             g_value_set_double(value, unicode_character_map_get_preview_size(charmap));
             break;
         default:
-            G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
+            G_OBJECT_WARN_INVALID_PROPERTY_ID(gobject, prop_id, pspec);
             break;
     }
     return;
@@ -1261,7 +1259,7 @@ unicode_character_map_class_init (UnicodeCharacterMapClass *klass)
     GObjectClass *object_class = G_OBJECT_CLASS(klass);
     GtkWidgetClass *widget_class = GTK_WIDGET_CLASS(klass);
 
-    object_class->finalize = unicode_character_map_finalize;
+    object_class->dispose = unicode_character_map_dispose;
     object_class->get_property = unicode_character_map_get_property;
     object_class->set_property = unicode_character_map_set_property;
 
@@ -1344,44 +1342,40 @@ unicode_character_map_class_init (UnicodeCharacterMapClass *klass)
     g_object_class_install_property(object_class,
                                     PROP_ACTIVE_CHAR,
                                     g_param_spec_uint("active-character",
-                                                        NULL, NULL,
+                                                        NULL,
+                                                        "Active character",
                                                         0,
                                                         UNICODE_UNICHAR_MAX,
                                                         0,
                                                         G_PARAM_READWRITE |
-                                                        G_PARAM_STATIC_NAME |
-                                                        G_PARAM_STATIC_NICK |
-                                                        G_PARAM_STATIC_BLURB));
+                                                        G_PARAM_STATIC_STRINGS));
 
     g_object_class_install_property(object_class,
                                     PROP_FONT_DESC,
                                     g_param_spec_boxed("font-desc",
-                                                        NULL, NULL,
+                                                        NULL,
+                                                        "PangoFontDescription",
                                                         PANGO_TYPE_FONT_DESCRIPTION,
                                                         G_PARAM_READWRITE |
-                                                        G_PARAM_STATIC_NAME |
-                                                        G_PARAM_STATIC_NICK |
-                                                        G_PARAM_STATIC_BLURB));
+                                                        G_PARAM_STATIC_STRINGS));
 
     g_object_class_install_property(object_class,
                                     PROP_PREVIEW_SIZE,
                                     g_param_spec_double("preview-size",
-                                                        NULL, NULL,
+                                                        NULL,
+                                                        "Preview size",
                                                         6, 96, 14,
                                                         G_PARAM_READWRITE |
-                                                        G_PARAM_STATIC_NAME |
-                                                        G_PARAM_STATIC_NICK |
-                                                        G_PARAM_STATIC_BLURB));
+                                                        G_PARAM_STATIC_STRINGS));
 
     g_object_class_install_property(object_class,
                                     PROP_CODEPOINT_LIST,
                                     g_param_spec_object("codepoint-list",
-                                                        NULL, NULL,
+                                                        NULL,
+                                                        "UnicodeCodepointList",
                                                         UNICODE_TYPE_CODEPOINT_LIST,
                                                         G_PARAM_READWRITE |
-                                                        G_PARAM_STATIC_NAME |
-                                                        G_PARAM_STATIC_NICK |
-                                                        G_PARAM_STATIC_BLURB));
+                                                        G_PARAM_STATIC_STRINGS));
 
     /* Keybindings */
     GtkBindingSet *binding_set = gtk_binding_set_by_class(klass);
@@ -1435,7 +1429,8 @@ unicode_character_map_class_init (UnicodeCharacterMapClass *klass)
 /**
  * unicode_character_map_new:
  *
- * Returns: (transfer full): a new #UnicodeCharacterMap
+ * Returns: (transfer full): A newly created #UnicodeCharacterMap.
+ * Free the returned object using #g_object_unref().
  */
 GtkWidget *
 unicode_character_map_new (void)
@@ -1483,7 +1478,7 @@ unicode_character_map_get_font_desc (UnicodeCharacterMap *charmap)
  * unicode_character_map_get_codepoint_list:
  * @charmap: a #UnicodeCharacterMap
  *
- * Returns: (transfer none) (nullable): the current codepoint list
+ * Returns: (transfer none) (nullable): The current #UnicodeCodepointList
  */
 UnicodeCodepointList *
 unicode_character_map_get_codepoint_list (UnicodeCharacterMap *charmap)
@@ -1524,12 +1519,12 @@ unicode_character_map_set_codepoint_list (UnicodeCharacterMap *charmap,
 /**
  * unicode_character_map_set_preview_size:
  * @charmap: a #UnicodeCharacterMap
- * @size: a double
+ * @size: new preview size
  *
  * Sets the preview size to @size.
  */
 void
-unicode_character_map_set_preview_size (UnicodeCharacterMap *charmap, double size)
+unicode_character_map_set_preview_size (UnicodeCharacterMap *charmap, gdouble size)
 {
     g_return_if_fail(UNICODE_IS_CHARACTER_MAP(charmap));
     priv->preview_size = size;
@@ -1543,7 +1538,7 @@ unicode_character_map_set_preview_size (UnicodeCharacterMap *charmap, double siz
  * unicode_character_map_get_preview_size:
  * @charmap: a #UnicodeCharacterMap
  *
- * Returns: the current preview size
+ * Returns: The current preview size
  */
 double
 unicode_character_map_get_preview_size (UnicodeCharacterMap *charmap)
@@ -1556,7 +1551,7 @@ unicode_character_map_get_preview_size (UnicodeCharacterMap *charmap)
  * unicode_character_map_get_active_character:
  * @charmap: a #UnicodeCharacterMap
  *
- * Returns: the currently selected character
+ * Returns: The currently selected #gunichar
  */
 gunichar
 unicode_character_map_get_active_character (UnicodeCharacterMap *charmap)
