@@ -2,7 +2,7 @@
  *
  * Originally a part of Gucharmap
  *
- * Copyright (C) 2017 - 2020 Jerry Casiano
+ * Copyright (C) 2017 - 2021 Jerry Casiano
  *
  *
  * Copyright © 2004 Noah Levitt
@@ -36,6 +36,15 @@
 #include "unicode-unihan.h"
 #include "unicode-scripts.h"
 
+/**
+ * SECTION: unicode-info
+ * @short_description: Unicode related utility functions
+ * @title: Unicode Information
+ * @include: unicode-info.h
+ *
+ * This widget provides a way to search for characters by name, value or
+ * other information.
+ */
 
 /* constants for hangul (de)composition, see UAX #15 */
 #define SBase 0xAC00
@@ -558,21 +567,22 @@ unicode_unichar_to_printable_utf8 (gunichar uc, gchar *outbuf)
   if (! unicode_unichar_validate (uc) || (! unicode_unichar_isgraph (uc)
       && g_unichar_type (uc) != G_UNICODE_PRIVATE_USE))
     return 0;
-  else if (g_unichar_type (uc) == G_UNICODE_SPACING_MARK
-      || g_unichar_type (uc) == G_UNICODE_ENCLOSING_MARK
-      || g_unichar_type (uc) == G_UNICODE_NON_SPACING_MARK)
-    {
-      gint x;
+  /* XXX : https://github.com/FontManager/font-manager/issues/143 */
+  //else if (g_unichar_type (uc) == G_UNICODE_SPACING_MARK
+      //|| g_unichar_type (uc) == G_UNICODE_ENCLOSING_MARK
+      //|| g_unichar_type (uc) == G_UNICODE_NON_SPACING_MARK)
+    //{
+      //gint x;
 
-      outbuf[0] = ' ';
-      outbuf[1] = '\xe2'; /* ZERO */
-      outbuf[2] = '\x80'; /* WIDTH */
-      outbuf[3] = '\x8d'; /* JOINER (0x200D) */
+      //outbuf[0] = ' ';
+      //outbuf[1] = '\xe2'; /* ZERO */
+      //outbuf[2] = '\x80'; /* WIDTH */
+      //outbuf[3] = '\x8d'; /* JOINER (0x200D) */
 
-      x = g_unichar_to_utf8 (uc, outbuf + 4);
+      //x = g_unichar_to_utf8 (uc, outbuf + 4);
 
-      return x + 4;
-    }
+      //return x + 4;
+    //}
   else
     return g_unichar_to_utf8 (uc, outbuf);
 }
