@@ -1,6 +1,6 @@
 /* unicode-search-bar.h
  *
- * Copyright (C) 2018 - 2020 Jerry Casiano
+ * Copyright (C) 2018 - 2021 Jerry Casiano
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -34,7 +34,8 @@ G_BEGIN_DECLS
 #define UNICODE_TYPE_SEARCH_BAR (unicode_search_bar_get_type())
 G_DECLARE_FINAL_TYPE(UnicodeSearchBar, unicode_search_bar, UNICODE, SEARCH_BAR, GtkSearchBar)
 
-UnicodeSearchBar * unicode_search_bar_new (void);
+GtkWidget * unicode_search_bar_new (void);
+void unicode_search_bar_set_character_map (UnicodeSearchBar *self, UnicodeCharacterMap *character_map);
 
 G_END_DECLS
 
